
#include <libc.h>
#include <i86.h>
#include <dirent.h>
#include <sys/psinfo.h>
#include <sys/osinfo.h>
#include <sys/sched.h>
#include <sys/prfx.h>
#include <sys/fd.h>
#include <sys/types.h>
#include <sys/disk.h>
#include "struct.h"
#include "extern.h"

/*********************************************************************************/
/*	Function hardware_init()                                                     */
/*                  -	This function is called from the main() during the 		 */
/*						initialization of the driver.  This function will 		 */
/*						preform any hardware specific operations before any      */
/*						open/close/read/write requests are processed.       	 */
/* 		Parameters Passed: none											         */
/*********************************************************************************/


int hardware_init( int argc, char *argv[], char *minorname )
{
	int		 i;
	int		 fd;
	char	*p,
			 nm[25];

/* one of the first things to do is to initialize the 'device' structure.
   Here we initialize the structure to NULL and then set as required.
   ** The device name is very important and must be unique for each device.
*/ 

	if( add_drive( 0, (-1), "", 0 ) != 0 )
	{
		fprintf( stderr, "no memory\n" );
		exit( 1 );
	}

	for( i = 1; optind < argc && i < numdevs; ++optind, ++i )
	{
		if( (p = strchr( argv[optind], '=' )) == NULL )
		{
			fprintf( stderr, "invalid disk specification: %s\n", argv[optind] );
			exit( 1 );
		}
		else
		{
			sprintf( nm, "%s%u", minorname, atoi(argv[optind]) );

			if( (fd = open(++p, O_RDONLY)) < 0 )
				perror( "open" );
			else if( add_drive( i, fd, nm, atoi(argv[optind]) ) >= 0 )
			{
				++device[0].stat.st_nlink;
				++device[0].stat.st_size;
				printf( "add %s\n", argv[optind] );
			}
			else
				printf( "duplicate device: %s\n", argv[optind] );
		}
	}	

	return( i );
}


int add_drive( int i, int fd, char *nm, int assignum )
{
	int				 n;

	for( n = 0;  n < i;  ++n )
	{
		if( device[i].stat.st_ino == assignum )
			return( -1 );
	}

	memset( &device[i], NULL, sizeof(struct device) );	/* initialize to NULL */
	strcpy( device[i].name, nm );
	device[i].namelen = strlen( nm );
	device[i].fd = fd;

	device[i].stat.st_ino      = assignum;
	device[i].stat.st_dev      =
		device[i].stat.st_rdev = (dev_t) ((getnid() << 16L) + (devno << 10L) + i); /* Major and Minor Device number w/node ID. */
	device[i].stat.st_nlink    = 1;				/* number of links.				*/  
	device[i].stat.st_mode     = S_IFDIR|S_IRUSR|S_IRGRP|S_IROTH|S_IXUSR|S_IXGRP|S_IXOTH;
	device[i].stat.st_status   = _FILE_USED;

	if( assignum >= 3 && fd >= 0 && block_read(fd, 1L, 1, buff) == 1 )
		get_dev_stat( &device[i] );
	else
	{
		device[i].type = _FLOPPY;
		device[i].stat.st_ftime = *timep;	/* Time of creation.			*/
		device[i].stat.st_atime = *timep;	/* Time of last access.			*/
		device[i].stat.st_mtime = *timep;	/* Time of last modification.	*/
		device[i].stat.st_ctime = *timep;	/* Time of last status change.	*/
	}

	return( assignum );
}
