
#include <libc.h>
#include <sys/sendmx.h>
#include <sys/kernel.h>
#include <sys/fd.h>
#include <sys/disk.h>

#include "struct.h"
#include "extern.h"
#include "define.h"
#include "proto.h"


int block_read_msg( pid_t pid )
{
	struct ocb			*ocb;
	struct _mxfer_entry	 mx_entry[3];
	long				 block;
	int					 num,
						 fd;
	
	fd = io_msg.block_read.fd;
	if( (ocb = get_ocb(pid, fd)) == (struct ocb *) (-1) )
		return(REPLY);
	
	if((check_access(pid, &(ocb->device->stat)) & S_IREAD) == 0)
	{
		io_msg.status = EPERM;
		return(REPLY);
	}

	num = min( MAX_BLOCKS, io_msg.block_read.num_blks );
	block = io_msg.block_read.block;

	if( (ocb->flags & OCB_RAW) == 0 )
	{
		io_msg.lseek.fd = fd;
		io_msg.lseek.whence = SEEK_SET;
		io_msg.lseek.offset = block * 512L;
		seek_msg( pid );
		if( io_msg.lseek_reply.status != EOK )
			return( REPLY );
		io_msg.lseek.fd = fd;
		io_msg.read.nbytes = num * 512L;
		return( read_msg( pid ) );
	}

	num = block_read( ocb->fp.device->fd, block, num, buff );

	if( num < 1 )
	{
		io_msg.status = errno;
		return(REPLY);
	}

	io_msg.block_read_reply.status = EOK;
	io_msg.block_read_reply.blks_read = num;
	_setmx( &mx_entry[0], &io_msg.block_read_reply, (sizeof(io_msg.block_read_reply) - 1) );
	_setmx( &mx_entry[1], buff, num * 512L );
	Replymx(pid, 2, &mx_entry);
	
	return(NO_REPLY);
}


int block_write_msg( pid_t pid )
{
	struct ocb			*ocb;
	long				 bytes;
	int					 num,
						 last;
	
	if( (ocb = get_ocb(pid, io_msg.block_write.fd)) == (struct ocb *) (-1) )
		return(REPLY);
	
	if((check_access(pid, &(ocb->device->stat)) & S_IWRITE) == 0)
	{
		io_msg.status = EPERM;
		return(REPLY);
	}

	if( (ocb->flags & OCB_RAW) == 0 )
	{
		io_msg.status = EOPNOTSUPP;
		return(REPLY);
	}

	io_msg.status = EROFS;				// read-only file system
	return(REPLY);

#if 0
	num = min( MAX_BLOCKS, io_msg.block_write.num_blks );
	bytes = Readmsg( pid, (sizeof(io_msg.block_write) - 1), buff, (num * 512L) );
	num = bytes / 512L;
	num = block_write( ocb->fp.device->fd, io_msg.block_write.block, num, buff );

	if( num > 0 )
	{
		last = io_msg.block_write.block + num - 1;
		if( ocb->fp.blk >= io_msg.block_write.block && ocb->fp.blk <= last )
		{
			bytes = (ocb->fp.blk - io_msg.block_write.block) * 512L;
			memcpy( ocb->fp.buff, &buff[bytes], 512 );
		}
		io_msg.block_write_reply.status = EOK;
		io_msg.block_write_reply.blks_written = num;
	}
	else
	{
		io_msg.block_write_reply.status = errno;
		io_msg.block_write_reply.blks_written = 0;
	}
	
	return(REPLY);
#endif
}
