
#include <libc.h>
#include <sys/fd.h>
#include <sys/disk.h>

#include "define.h"
#include "struct.h"
#include "extern.h"

/******************************************************************************/
/*	Function open_msg() - 	This function is called when the IO_OPEN message  */
/*							is received.  This function will check access 	  */
/*							rights of the application opening the driver as   */
/*							well as make sure the driver is not already open. */
/*							It will then associate a OCB with the file 		  */
/*							descriptor which is sent with the message. 		  */
/* 		Parameters Passed:													  */
/*				pid :	Process ID of calling application.					  */
/******************************************************************************/

int open_msg( pid_t pid, int force_perm )
{
	mode_t		 mode;
	int			 index;
	struct ocb  *ocb;
	char		*node,
				*dir,
				*fname,
				*ext,
				*nm,
				 flags = 0;

	_splitpath2( io_msg.open.path, buff, &node, &dir, &fname, &ext );
#ifdef SUPPORT_RAW
	if( strncmp( rawname, dir, rawlen ) == 0 && dir[rawlen] == '/' && dir[rawlen+1] )
	{
		flags |= OCB_RAW;
		dir += RAW_LEN;					// offset the length of rawname prefix
	}
#endif

	nm = io_msg.open.path;
	if( node != NULL && *node )
		nm += (strlen(node) + 1);
	if((index = lookup_name(nm)) == -1)	// go see if this name exists
	{
		io_msg.open_reply.status = ENOENT;
		return(REPLY);
	}

	if( force_perm == FALSE )
	{
#if 0
ifdef READ_ONLY_FSYS
		if( (io_msg.open.oflag & O_ACCMODE) == O_WRONLY ||
						(io_msg.open.oflag & O_ACCMODE) == O_RDWR )
		{
			io_msg.open_reply.status = EROFS;	// read-only file system
			return(REPLY);
		}
#endif
	
		if(((mode=check_access(pid, &device[index].stat)) & S_IRWXU) == 0)
		{
			io_msg.open_reply.status = EPERM;
			return(REPLY);
		}
	}

	if( pid == (-1) )
	{
		memset( ocb = &dummy, 0, sizeof(struct ocb) );
	}
	else if( (ocb = (struct ocb *) calloc(1, sizeof(struct ocb))) == NULL )
	{
		io_msg.open_reply.status = ENOMEM;
		return(REPLY);
	}

	++ocb->count;
	ocb->flags = flags;
	ocb->fp.device = ocb->device = &device[index];
	ocb->mode = mode;					/* Save access mode bits. */
	ocb->fp.fattr = Q2_DIRECTORY;		// by definition, the top lvl is a dir
	ocb->fp.count = 1;
	ocb->fp.ffirst_xtnt = 1;
	memcpy( &ocb->fstat, &ocb->device->stat, sizeof( ocb->fstat ) );

	if( index == 0 )					// top level directory
	{
		ocb->fp.fnum_xtnt = 1;
		ocb->device = &device[1];		// devices for top level are other devs
	}
	else if( flags & OCB_RAW )
	{
		ocb->fp.fattr &= ~Q2_DIRECTORY;	// raw below top are not directories
	}
	else
	{
		++ocb->fp.lvl;
		if( (io_msg.open_reply.status = q2_open(ocb,io_msg.open.path)) != EOK )
		{
			if( pid != (-1) )
				free( ocb );
			return(REPLY);
		}
	}

	if( pid != (-1) && map_ocb(pid, io_msg.open.fd, ocb) )
	{
		free( ocb );
		io_msg.open_reply.status = errno;
		return(REPLY);
	}

	io_msg.open_reply.status = EOK;
	return(REPLY);
}


int q2_open( struct ocb *ocb, char *nm )
{
	short			 tmp;
	char		 	*p;
	struct dir_entry dir_entry;

	strtok( p = nm, "/" );

	if( (ocb->fp.device->type & _FLOPPY) && get_dev_stat(ocb->fp.device) )
		return( ENOENT );

	if( q2_fget( buff, 0x44 - sizeof(struct xtnt_hdr), &ocb->fp ) == 0 )
		return( ENOENT );

	q2_fget( &dir_entry, sizeof(struct dir_entry), &ocb->fp );
	ocb->fp.ffirst_xtnt = dir_entry.ffirst_xtnt;
	ocb->fp.fnum_xtnt = dir_entry.fnum_xtnt;
	ocb->fp.fattr = dir_entry.fattr;
	ocb->fp.blk = 0L;

	while( p = strtok(NULL, "/") )
	{
		if( ! (dir_entry.fattr & Q2_DIRECTORY) )
			return( ENOTDIR );

		++ocb->fp.lvl;
		q2_fget(&tmp, sizeof(short), &ocb->fp);
		q2_fget(&tmp, sizeof(short), &ocb->fp);

		if( tmp & 0x8000 )
			q2_fget(&tmp, sizeof(short), &ocb->fp);

		while( q2_fget(&dir_entry, sizeof(dir_entry), &ocb->fp) == sizeof(dir_entry) )
		{
			if( dir_entry.fstat & Q2_USED )
			{
				if( ! strcmp(p, dir_entry.fname) )
				{
					ocb->fp.ffirst_xtnt = dir_entry.ffirst_xtnt;
					ocb->fp.fnum_xtnt = dir_entry.fnum_xtnt;
					ocb->fp.fattr = dir_entry.fattr;
					ocb->fp.blk = 0L;
					break;
				}
			}
		}
		if( ocb->fp.blk )
			return( EACCES );
	}

	load_stat( ocb->fp.device->stat.st_dev, &ocb->fstat, &dir_entry );

	return( EOK );
} 	
