
/* q2fsys Input/Output Manager */

#ifdef __USAGE		/* q2fsys main.c */
%C - Sample Resource Manager that shows how to register multiple devices

%C [-d name] [-m minor] [-n #] [-p #] [-V] q2dsk=qnx4dev [...] &
Options:
 -d name          Device name to register (default: /q2).
 -m minor         Minor device name to register (default: "").
                     This will create devices /q2/1, /q2/2, etc.
 -n #             Number of minor devices to register (default: 4).
 -p #             Priority (default: 15).
 -v               Run verbose. Output to stderr. (-VV is very verbose)
 q2dsk=qnx4dev    q2 disk (1,2,3,...) = qnx4 device (/dev/fd0,/dev/hd0t7,...)
 [...]            additional disk specifications
#endif

#include <libc.h>
#include <dirent.h>
#include <sys/kernel.h>
#include "struct.h"
#include "define.h"

union msg		 io_msg;				/* union of all msg types to driver */
time_t __far	*timep;
void			*fd_ctrl;
int				 devno;
int				 rawlen;
char			 rawname[MAX_NAMELN];	 /* name to register at top of raw tree */
char			 devicename[MAX_NAMELN]; /* name to register at top of tree */

struct ocb		 dummy;
struct device device[MAX_DEVS] = {0};
struct dirent dirbuf[_DIRBUF];
char buff[ (MAX_BLOCKS*512L) ];

short int		 search  = 0;			/* search (as opposed to assign) */
short int		 verbose = 0;			/* verbose level */
short int		 numdevs = 4;			/* number of minor devices */


void main( int argc, char *argv[] )
{
	pid_t pid;
	char reply;
	int opt;
	char minorname[11];
	int priority;

	/* some defaults */
	priority = 15;
	strcpy( rawname, RAW_NAME );
	strcpy( devicename, DEVICE_NAME );
	strcpy( minorname,  MINOR_NAME );

	while( (opt = getopt( argc, argv, "d:p:m:n:vV" )) != -1 )
	{
		switch( opt )
		{
			case 'd':	strncpy( devicename, optarg, (MAX_NAMELN - 1) );
						break;
			case 'm':	strncpy( minorname, optarg, 8 );
						break;
			case 'p':	priority = atoi( optarg );
						break;
			case 'n':	numdevs = atoi( optarg );
						if( numdevs < 1 || numdevs > MAX_DEVS )
						{
							fprintf( stderr, "Invalid number of devices. Must be between 1 and %d\n", MAX_DEVS );
							exit( 1 );
						}
						break;
			case 'S':	++search;
						break;
			case 'v':
			case 'V':	++verbose;
						break;
			default:	break;
		}
	}

	if( *devicename != '/' )
		strcat( rawname, "/" );
	strcat( rawname, devicename );
	rawlen = strlen( rawname );
	if( rawname[rawlen-1] == '/' )
		rawname[--rawlen] = 0;
	manager_init( priority );			/* initialize the resource manager */

	/* Entry point #1 */
	/*	- use this function to perform any initialization of hardware */
	numdevs = hardware_init( argc, argv, minorname );

	for( ;; )							/* Message processing loop. */
	{
		pid = Receive( 0, &io_msg, sizeof(io_msg) );

			/* After receiving message call function to process it if it is	 */
			/* supported.  Reply with ENOSYS if not supported.				 */
			/* 'pid' is the process id of the process that sent us a message */

		if( verbose )
		   print_event( io_msg.type, pid );

		switch( io_msg.type )
		{
			case _SYSMSG:				/* from 'sin ver' */
				reply = sys_msg( pid );
				break;

			case _IO_HANDLE:
				reply = handle_msg( pid );
				break;

			case _IO_OPEN:				/* from an open() call */
				reply = open_msg( pid, FALSE );
				break;

			/* a close request. Either from a close() call or from termination */
			/* of the program. The Operating System ensures that even if a */
			/* client dies abnormally, if it has a file descriptor open to */
			/* your manager, your program will ALWAYS receive a close message. */
			/* Note that this is NETWORK TRANSPARENT.. it works both for */
			/* locally opened devices and devices opened across the network. */
			case _IO_CLOSE:
				reply = close_msg( pid );
				break;

			case _IO_FSTAT:				/*  file status request */
				reply = fstat_msg( pid );
				break;

			case _IO_STAT:				/* status request */
				reply = stat_msg( pid );
				break;

			case _IO_QIOCTL:			/* from a qnx_ioctl() call */
				reply = ioctl_msg( pid );
				break;

			case _IO_DUP:				/* from a ddup() call */
				reply = dup_msg( pid );
				break;

			case _IO_CHMOD:				/* from a chmod() call */
				reply = chmod_msg( pid );
				break;

			case _IO_CHOWN:				/* change ownership */
				reply = chown_msg( pid );
				break;

			case _IO_UTIME:				/* update the device access time */
				reply = utime_msg( pid );
				break;

			case _IO_CHDIR:
				reply = chdir_msg( pid );
				break;

			case _IO_READDIR:
				reply = readdir_msg( pid );
				break;

			case _IO_REWINDDIR:
				reply = rewinddir_msg( pid );
				break;

			case _IO_LSEEK:				/* seek to a specified offset */
				reply = seek_msg( pid );
				break;

			case _IO_READ:				/* read some data */
				reply = read_msg( pid );
				break;

			case _IO_WRITE:				/* write some data */
#ifdef NOTDONE
				reply = write_msg( pid );
#else
				io_msg.status = EOPNOTSUPP;
				reply = REPLY;
#endif
				break;

#ifdef SUPPORT_RAW
			case _FSYS_BLOCK_READ:		/* read some block data */
				reply = block_read_msg( pid );
				break;

			case _FSYS_BLOCK_WRITE:		/* write some block data */
				reply = block_write_msg( pid );
				break;
#endif

			/* these message types are currently not supported */
			case _IO_RENAME:
			case _IO_GET_CONFIG:
			case _IO_FLAGS:
			case _IO_LOCK:
				io_msg.status = EOPNOTSUPP;
				reply = REPLY;
				break;
			default:
				io_msg.status = ENOSYS;
				reply = REPLY;
				break;
		}
		if( reply )
			Reply( pid, &io_msg, sizeof(io_msg) );
	}
}


char	*evtype[] =
{
	"unknown",
	"_IO_OPEN",
	"_IO_CLOSE",
	"_IO_READ",
	"_IO_WRITE",
	"_IO_LSEEK",
	"_IO_RENAME",
	"_IO_GET_CONFIG",
	"_IO_DUP",
	"_IO_HANDLE",
	"_IO_FSTAT",
	"_IO_CHMOD",
	"_IO_CHOWN",
	"_IO_UTIME",
	"_IO_FLAGS",
	"_IO_LOCK",
	"_IO_CHDIR",
	"unknown",
	"_IO_READDIR",
	"_IO_REWINDDIR",
	"_IO_IOCTL",
	"_IO_STAT",
	"_IO_SELECT",
	"_IO_QIOCTL"
} ;

void print_event( int msgtype, pid_t pid )
{
	fprintf( stderr, "Received message from %d. Type = 0x%x, %s\n",
		pid, msgtype,
		evtype[ msgtype - 0x0100 ] );
}
