#include <stdlib.h>
#include <signal.h>
#include <i86.h>
#include <stddef.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/psinfo.h>
#include <sys/osinfo.h>
#include <sys/sched.h>
#include <sys/prfx.h>
#include <sys/fd.h>
#include <sys/types.h>
#include "struct.h"
#include "extern.h"

/*********************************************************************************/
/*	Function manager_init()                                                      */
/*                  -	This function is called from the main() during the 		 */
/*						initialization of the driver.  This function will 		 */
/*						register this application as a device driver, notify the */
/*						kernel that we are a server so that	we will receive 	 */
/*						system messages.										 */
/* 		Parameters Passed: int priority                                          */
/*********************************************************************************/
void manager_init(int priority)
{
   int i;
   struct _osinfo	osdata;

	setprio(0, priority);		/* Boost our priority.			*/

   /* Allow only SIGTERM for graceful termination, ignore all others. */
	for( i = _SIGMIN; i <= _SIGMAX; ++i) /* Ignore signals all signal except SIGTERM. */
		if( i != SIGTERM)
			signal( i, SIG_IGN);

	signal( SIGTERM, &terminate );	/* Setup the terminate signal handler function. */

	if(qnx_pflags(~0, _PPF_SERVER, NULL, NULL)) /* Get system messages. */
	{
		perror("pflags");
		exit(-1);
	}

	if((devno = qnx_device_attach()) == -1) /* Attach a device number to our I/O manager */
	{
		perror("qnx_device_attach");
		exit(-1);
	}

	if (qnx_osinfo(0, &osdata) == -1)       
	{
	   perror("qnx_osinfo");
		exit(-1);
	}  

    /* Get a pointer to system time segment. */
	timep = MK_FP(osdata.timesel, offsetof(struct _timesel, seconds));

    /* In this manager, we will be attaching several devices under our
       prefix (up to MAX_DEV devices).
     */
    if ( verbose )
       fprintf(stderr,"Registering prefix %s\n",devicename);
#ifdef SUPPORT_RAW
	if(qnx_prefix_attach(rawname, NULL, 0))  /* Attach prefix for /dev/q2 */
	{
		perror("qnx_prefix_attach");
		exit(-1);
	}
#endif
	if(qnx_prefix_attach(devicename, NULL, 0))  /* Attach prefix for /q2 */
	{
		perror("qnx_prefix_attach");
		exit(-1);
	}
	if(!(fd_ctrl = __init_fd(getpid())))
	{
		perror("__init_fd");
		exit(-1);
	}
}

