
#ifdef __USAGE		/* q2chkfsys main.c */
%C - Check a QNX 2 file system for integrity

%C - [-srpfv] [-f path] [-b bad_block_file] drive
Options:
 -f               do not Fix anything
 -r               Rebuild the bitmap WITHOUT error messages
 -p               do not Pause
 -s               Suppress stats display
 -v               Suppress verbose display
#endif

/*********************************************************************

	$Header: [60]3:/src/util/QRCS/chkfsys.c.v 2.20 Wed Feb 17 14:38:57 1993 randy Exp $

	$Log: 	[60]3:/src/util/QRCS/chkfsys.c.v $
	
	Revision 2.20 Wed Nov 2  14:38:57 1994 terry
	Ported to QNX4.2 to check QNX2 file systems.
	
	Revision 2.20 Wed Feb 17 14:38:57 1993 randy
	2.20
	
	Revision 2.15 Tue Feb 16 18:26:13 1993 randy
	2.15
	
	Revision 1.6 Mon Jun 12 17:16:09 1989 dtdodge
	Fixed a bug where a extended dir entry was moved via relink file to
	below 32Meg. Chksfsys tried to make it a old style dir (4 bytes not 6)
	header.
	
	Revision 1.5 7-Mar-89 2:06:43pm aboyd
	now prints bitmap error regardless of -p
	
	Revision 1.4 30-Sep-88 2:45:56pm jeff
	Changed Warning Message to improve clarity.
	
	Revision 1.3 22-Aug-88 10:07:38am danh
	Fixed %l arguments to printf to read %ld.
	
	Revision 1.2 9-Aug-88 3:37:32pm grant
	Changes added to support partitions > 256 Meg. Changes were made by 
	Dan Frank of ACT-PC.
	
	Revision 1.1 9-Aug-88 3:30:13pm grant
	 *** QRCS - Initial Revision ***
	
**********************************************************************/

/*
 * List of fixes we can do on the fly (someday).
 * The only ones I trust are ones where the xtnt info disagees with
 * the dir entry. In this case the xtnt hdrs should be believed.
 * Corruption in the xtnt hdrs probably indicates garbage with no
 * possible fix.
 *
 * previous xtnt wrong - fix it so it points back correctly
 * last xtnt does not agree with dir entry - fix dir entry  **
 * num xtnts does not agree with dir entry - fix dir entry  **
 * num blks  does not agree with dir entry - fix dir entry  ** 
 * bound does not match size of xtnt - fix bound (user must rerun chkfsys)
 * file uses a blk in use - warn user to copy file to floppy then zap file
 */

#include <stdio.h>
#include "struct.h"


/*
 * Too macros for getting at the lo and hi words of a long.
 */

struct
{
	unsigned lo;
	unsigned hi;
} ;

#define hi(blk) (blk.hi)
#define lo(blk) (blk.lo)

#define FIX_LAST_XTNT	0
#define FIX_NUM_XTNTS	1
#define FIX_NUM_BLKS	2

extern unsigned My_nid;
extern unsigned char Release_num;

char		 path[346];					/* Good for 20 levels of 16 char names */
char		*fname;						/* Last component of pathname for a file */
char		*bfile;
char		 fname_buf[40];
char		 pauseflag = 1;
char		 fixflag = 1;
char		 verboseflag = 1;
char		*fequals;
char		 bufhdr[sizeof(struct r_w_msg)];
char		 buf[512];
int			 drive;
unsigned	 nid;
int			 level;
unsigned	 nxtnts;
long		 blk, nblks;
long		 disk_blks;
long		 bitmap_bytes, bitmap_blks;
unsigned	*old_bitmap_seg, *new_bitmap_seg;
unsigned	 status;

char		 nostats, bitmap_smaller, rebuild, supress_error;

char		 small_blks;				/* Small fsys <= 32 Meg */
char		 small_xtnts;				/* Small xtnts in fsys <= 64K */

/*
 * Keep all statistics as longs. The sumfiles and sumdirs could be kept
 * as short unsigned's but why bother..
 */
long		 sumblks, sumxtnts, sumfiles, sumdirs;
long		 maxblks, maxxtnts, maxfiles;

int			 dp;


main( char argc, char *argv[] )
{
	char		*p;
	char		 arg;
	int			 i ;
	int			 opt;
	unsigned	 paras ;
	long		 c;
	long		 n;
	FILE		*fp = 0;
	struct dir_entry dir_buf;
	struct ldisk_entry disk_buf;

	fname = "";
	nid = My_nid;

	while( (opt = getopt( argc, argv, "sfprvb:f:" )) != -1 )
	{
		switch( opt )
		{
			case 's':	++nostats;
						break;
			case 'f':	fixflag = 0;
						break;
			case 'p':	pauseflag = 0;
						break;
			case 'r':	++rebuild;
						break;
			case 'v':	verboseflag = 0;
						break;
			case 'b':	fp = fopen( bfile = optarg, "r" );
						break;
			case 'f':	fequals = optarg;
			default:	break;
		}
	}

	if( optind >= argc )
	{
		fprintf( stderr, "%s: no drive specified\n", argv[0] );
		exit( 1 );
	}
	drive = argv[optind++];

	dp = open( drive, O_RDWR );

	if(fp == 0)
	{
		strcat(strcpy(fname_buf, argv[1]), ":/bad_blks");
		fp = fopen(bfile = fname_buf, "r");
	}

	if(pauseflag)
	{
		printf("**WARNING** CHKFSYS should be run when the file system being checked is idle.\n");
		printf("If any file is open for write on this drive the rebuilt bitmap will be\n");
		printf("incorrect. If this bitmap is written back to the disk it will result in file\n");
		printf("system corruption.  Do not continue if there are any files on this drive\n");
		printf("which are open at this time.\n");
		query("Continue CHKFSYS to check the file system on this drive? (y/n) ");
	}
	dp = open( drive, O_RDWR );
	if( dp == (-1) )
	{
		perror( drive );
		fprintf( stderr, "%s: unable to open drive\r\l", argv[0] );
		exit( 1 );
	}

	block_read(dp, 1L, 1, &bufhdr);	/* Force a remount */

	if(block_read(dp, (long)Q2_BITMAP_BLK, 1, &bufhdr) != 0)
		abort("Unable to read first block of bitmap (BLK 2).\n");

	disk_get_entry(dp, &disk_buf);
	lo(disk_blks) = disk_buf.num_sctrs;
	if((hi(disk_blks) = disk_buf.ext_num_sctrs) == 0)
		++small_blks;

	if((disk_buf.disk_flags & DISK_LARGE_EXTENTS) == 0)
		++small_xtnts;
	if((disk_buf.disk_flags & DISK_LARGE_BLOCKS) == 0)
		++small_blks;
	if((disk_buf.disk_flags & DISK_LARGE_FSYS) == 0)
		small_blks = small_xtnts = 1;

	bitmap_bytes = ((disk_blks - 1) >> 3) + 1;	/* Does not include XTNT_HDR */

	if(bitmap_bytes != buf.size_xtnt)
	{
		if(bitmap_bytes > buf.size_xtnt)
		{
			bitmap_smaller = 1;
			printf("\007Warning: Bitmap on disk is smaller than the mounted disk size.\n");
			query("Continue? ");
		}
		else
			abort("Bitmap on disk is larger than the mounted disk size.\n");
		bitmap_bytes = buf.size_xtnt;
	}

	bitmap_blks = ((bitmap_bytes + Q2_XTNT_HDR - 1) >> 9) + 1;
	if(bitmap_blks != buf.bound_xtnt + 1)
		abort("Corrupt extent header for bitmap.\n");

	/* Allocate pointer arrays */

	if ((old_bitmap_seg = malloc((bitmap_blks / 128 + 1) * sizeof(unsigned)))
		== 0)
		abort("CHKFSYS: Out of memory\n");

	if ((new_bitmap_seg = malloc((bitmap_blks / 128 + 1) * sizeof(unsigned)))
		== 0)
		abort("CHKFSYS: Out of memory\n");

	/* Now fill the arrays with selectors of enough allocated */
	/* extra segments to hold the entire bitmap */

	for (n = bitmap_blks * 32, i = 0 ; n != 0 ; i++)
	{
		paras = (n < 4096 ? n : 4096) ;
		if ((old_bitmap_seg[i] = alloc_segment(paras)) == 0)
			abort("CHKFSYS: Out of memory\n");
		if ((new_bitmap_seg[i] = alloc_segment(paras)) == 0)
			abort("CHKFSYS: Out of memory\n");
		n -= paras ;
	}
			

	/* Initialize the memory for the new bitmap we will build */

	i = 0 ;
	set_extra_seg(new_bitmap_seg[i]);

	/* The extent header is sure to fall in the first segment */

	for(p = 0 ; p < Q2_XTNT_HDR ; ++p)
		@p = buf[(unsigned) p];

	c = Q2_XTNT_HDR ;

	/* The following code relies on the pointer to roll over. */
	/* Change this if we go to long pointers, like on the 386. */

	for(p = Q2_XTNT_HDR ; c < bitmap_bytes + Q2_XTNT_HDR ; ++p, ++c)
	{
		if (p == 0)		/* Rollover?  Go to the next segment. */
			set_extra_seg(new_bitmap_seg[++i]) ;
		@p = 0;
	}

	/* Good assumption:  our extra segment points to the last seg */

	while(((unsigned) p) % FBLK_SIZE)
		@p++ = 0xff;


	/* Now copy the old bitmap to memory */

	for(c = 0, i = 0 ; c < bitmap_blks ; ++c)
	{
		if(block_read(dp, (long)(Q2_BITMAP_BLK + c), 1, &bufhdr) != 0)
			error("Unable to read block %ld of the bitmap.\n", c + 1);

		if ((c % 128) == 0)		/* First block of a segment */
			set_extra_segment(old_bitmap_seg[i++]) ;

		for(p = 0 ; p < FBLK_SIZE ; ++p)
			@(p + ((c % 128) * FBLK_SIZE)) = buf[(unsigned) p];
	}

	set_bits(1L, 1L);
	if(fp)
	{
		supress_error = 1;
		sprintf(path, "Input from b=%s", bfile);
		while(fscanf(fp, "%lx", &blk) == 1)
			set_bits(blk, 1L);
		fclose(fp);
		supress_error = 0;
	}

	block_read(dp, 1L, 1, &bufhdr);
	p = &buf[sizeof(struct xtnt_hdr) + sizeof(struct dir_xtnt) + sizeof(struct dir_entry)];
	copy(&dir_buf, p, sizeof(struct dir_entry));
	if(Release_num >= 20)
		sprintf(path, "[%d]%1X:", nid, drive);
	else
		sprintf(path, "%1X:", drive);
	if(chk_xtnts(&dir_buf) == 1)
		abort("Corrupt disk root directory.\n");

	if(fequals)
	{
		strcpy(path, argv[1]);
		strcat(path, ":");
		strcat(path, fequals);
		fp = fopen(path, "rqv");
		get_dir_entry(fp, &dir_buf);
		printf("Checking %s\n", path);
		chk_xtnts(&dir_buf);
		exit(status);
	}

	ws("", 1L, 1, dir_buf.ffirst_xtnt);

	if(nostats == 0)
	{
		printf("\n           Directories     Files        Extents      Blocks\n");
		printf("Totals    %4ld/disk    %6ld/disk    %4ld/disk   %6ld/disk\n",
						sumdirs, sumfiles, sumxtnts, sumblks);
		printf("Maximums               %6ld/dir     %4ld/file   %6ld/file\n",
						maxfiles, maxxtnts, maxblks+1);
		printf("Averages               %6ld/dir     %4ld/file   %6ld/file\n",
						sumfiles/sumdirs, sumxtnts/sumfiles, sumblks/sumfiles);
	}

	chk_zapped_blks();
	exit(status);
}


ws( char *dname, long parent_xtnt, unsigned dir_index, long my_blk )
{
	struct dir_entry dir_buf;
	FILE *fp;
	int len, nfiles = 0, index;
	long n = 0;

	len = strlen(path);
	strcat(strcat(path, "/"), dname);
	fp = fopen(path, "rvq");
	if(level++ == 0)
		path[len] = '\0';

	fget(&lo(n), 2, fp);
	fget(&index, 2, fp);
	if(index & 0x8000)					/* Large Fsys Directory */
	{
		fget(&hi(n), 2, fp);
		dir_index |= 0x8000;
	}

	if(parent_xtnt & 0xffff0000)
		dir_index |= 0x8000;

	if(index != dir_index)
	{
		error(2,"Directory back link (dir_index) is incorrect.\n");
		if(fixflag)
		{
			block_read(dp, my_blk, 1, &bufhdr);
			(&buf[Q2_XTNT_HDR])->dir_index = dir_index;
			fix_xtnt_header(my_blk);
		}
	}

	if(n != parent_xtnt)
	{
		error(2,"Directory back link (parent_xtnt) is incorrect.\n");
		if(fixflag)
		{
			block_read(dp, my_blk, 1, &bufhdr);
			(&buf[Q2_XTNT_HDR])->parent_xtnt = lo(parent_xtnt);
			if(parent_xtnt & 0xffff0000)
				(&buf[Q2_XTNT_HDR])->ext_parent_xtnt = hi(parent_xtnt);
			fix_xtnt_header(my_blk);
		}
	}

	index = 0;
	while(fget(&dir_buf, sizeof(struct dir_entry), fp) == sizeof(struct dir_entry))
	{
		if(dir_buf.fstat & Q2_USED)
		{
			++nfiles;
			fname = dir_buf.fname;
			if(dir_buf.fattr & _DIRECTORY)
			{
				if(verboseflag)
					printf("Directory %s/%s\n", path, fname);
				if(chk_xtnts(&dir_buf) == 0)
					ws(dir_buf.fname, my_blk, index, dir_buf.ffirst_xtnt);
			}
			else
			{
				if(verboseflag)
					printf("\t File %s/%s\n", path, fname);
				chk_xtnts(&dir_buf);
			}
		}
		++index;
	}

	fclose(fp);
	--level;
	++sumdirs;
	if(maxfiles < nfiles)
		maxfiles = nfiles;
	path[len] = '\0';
}



chk_xtnts( struct dir_entry *dir_ptr )
{
	register struct dir_entry *dp = dir_ptr;
	long prev_xtnt = 0;

	if(dp->ffirst_xtnt == 0  ||  dp->flast_xtnt == 0)
	{
		error(1,"Corrupt directory entry. First or last extent (ffirst_xtnt flast_xtnt) is zero.\n");
		return(-1);
	}

	if(nostats == 0)
		stats(dp);

	blk = dp->ffirst_xtnt;
	nxtnts = nblks = 0;

	for(;;)
	{
		if(block_read(dp, blk, 1, &bufhdr) != 0)
		{
			error(1,"Physical read error attempting to read block.\n");
			return(-1);
		}

		if(chk_xtnt_hdr(blk, blk == dp->ffirst_xtnt, buf))
			return(-1);

		if(prev_xtnt != buf.prev_xtnt)
		{
			error(2,"Corrupt extent header. Previous extent pointer (prev_xtnt) incorrect.\n");
			if(fixflag)
			{
				buf.prev_xtnt = prev_xtnt;
				fix_xtnt_header(blk);
			}
		}

		set_bits(blk, buf.bound_xtnt + 1);

		++nxtnts;
		nblks += buf.bound_xtnt + 1;
		if(buf.next_xtnt == 0)
			break;

		prev_xtnt = blk;
		blk = buf.next_xtnt;
	}

	if(blk != dp->flast_xtnt)
	{
		error(2,"Corrupt directory entry. Last extent pointer (flast_xtnt) incorrect.\n");
		if(fixflag)
		{
			fix_dir_entry(FIX_LAST_XTNT, blk);
			dp->flast_xtnt = blk;
		}
	}

	if(nxtnts != dp->fnum_xtnt)
	{
		error(2,"Corrupt directory entry. Number of extents (fnum_xtnt) incorrect.\n");
		if(fixflag)
		{
			fix_dir_entry(FIX_NUM_XTNTS, (long)nxtnts);
			dp->fnum_xtnt = nxtnts;
		}
	}

	if(--nblks != dp->fnum_blks)	/* The following kludge will catch fsys problem */
	{
		error(2,"Corrupt directory entry. Number of blocks (fnum_blks) incorrect.\n");
		if(fixflag)
		{
			fix_dir_entry(FIX_NUM_BLKS, nblks);
			dp->fnum_blks = nblks;
		}
	}

	return(0);
}


chk_xtnt_hdr( long blk, int first_blk, char *blk_ptr )
{
	register char *bp = blk_ptr;
	long n;

	if(small_blks && (hi(bp->prev_xtnt) || hi(bp->next_xtnt) || hi(bp->bound_xtnt)))
	{
		error(3,"Corrupt extent header. Non-zero extent extension.\n");
		return(-1);
	}
	if(small_xtnts && hi(bp->size_xtnt))
	{
		error(3,"Corrupt extent header. Size of xtnt > 65535.\n");
		return(-1);
	}

	if(!first_blk  &&  bp->prev_xtnt == 0  &&  bp->next_xtnt == 0)
	{
		error(3,"Corrupt extent header. Previous or next extent (prev_xtnt next_xtnt) zero.\n");
		return(-1);
	}

	if(bp->next_xtnt  &&  bp->size_xtnt == 0)
	{
		error(3,"Corrupt extent header. Size of extent (size_xtnt) is zero.\n");
		return(-1);
	}

	if((n = (bp->size_xtnt + Q2_XTNT_HDR - 1) >> 9)  != bp->bound_xtnt) 
	{
		if(!(bp->next_xtnt == 0  &&  (bp->size_xtnt & 0x1ff) == 0x1f0  &&  bp->bound_xtnt == n + 1))
		{
			error(2,"Corrupt extent header. Bound of extent inconsistant with size of extent.\n");
			if(fixflag)
			{
				bp->bound_xtnt = n;
				fix_xtnt_header(blk);
			}
		}
	}

	return(0);
}


/*
 * Set some bits in the bitmap.
 */

set_bits( long blk, long numblks )
{
	long boffset ;
	unsigned seg ;
	char *offset;
	char bit;

	if(blk == 0)
	{
		error(1,"Corrupt extent header or bad input in b=file (extent block 0).\n");
		return(-1);
	}
	--blk;

	while(numblks)
	{
		--numblks;
		boffset = (blk >> 3);
		if(boffset >= bitmap_bytes)
		{
			error(1,"Corrupt extent header or b=file. Bound of extent (bound_xtnt) past end of disk.\n");
			return(-1);
		}
		boffset += Q2_XTNT_HDR;
		bit = 1 << (blk & 0x7);

		seg = (unsigned)((boffset & 0xffff0000) >> 16) ;/* Which segment is it? */
		set_extra_seg(old_bitmap_seg[seg]);
		offset = (char *)(boffset & 0x0000ffff) ;		/* Offset within seg */

		if(rebuild == 0  &&  (@offset & bit) == 0  &&  supress_error == 0)
			error(4,"Corrupt file. File uses a block marked as free in the bitmap.\n");

		set_extra_seg(new_bitmap_seg[seg]);
		if((@offset & bit) != 0)
			error(3,"Corrupt file. File uses a block which is already in use by another file.\n");
		@offset |= bit;
		++blk;
	}

	return(0);
}


stats( struct dir_entry *dir_ptr )
{
	struct dir_entry *dp = dir_ptr;

	if(maxblks < dp->fnum_blks)
		maxblks = dp->fnum_blks;
	if(maxxtnts < dp->fnum_xtnt)
		maxxtnts = dp->fnum_xtnt;

	sumblks += dp->fnum_blks + 1;
	sumxtnts += dp->fnum_xtnt;
	++sumfiles;
}


/*
 * This routine (as elsewhere) assumes that there are less than 0xffff
 * blocks in the bitmap. A very reasonable assumption.
 * It also makes the less reasonable assuption that the bitmap is
 * less than 0xffff bytes which corrosponds to a 256 Meg drive.
 * This may have to be fixed....
 * And was, by Dan Frank of ACT/PC.
 */

chk_zapped_blks()
{
	register char *p;
	long lp, j ;
	unsigned seg ;
	char bit, old_byte, new_byte;
	long zapped_blks = 0, missing_blks = 0 ;
	int i ;

	/*
	 * First set end bits in last byte in bitmap
	 * A perfect fit (bit == 0xff) is a special case in which we only set
	 * the last bit. In reality ALL bits are free. We use the bit to stop
	 * scans through the bitmap.
	 */

	lp = bitmap_bytes + (Q2_XTNT_HDR - 1);
	p = (char *)(lp & 0x0000ffff) ;
	seg = (unsigned)((lp & 0xffff0000) >> 16) ;
	if(bitmap_smaller)					/* Don't know exact # of blks ... Only within 8 */
	{
		set_extra_seg(old_bitmap_seg[seg]);
		bit = @p;
		set_extra_seg(new_bitmap_seg[seg]);
		@p |= bit;
	}
	else
	{
		set_extra_seg(new_bitmap_seg[seg]);
		bit = 0xff << (disk_blks % 8);
		@p |= (((unsigned)bit) == 0xff) ? 0x80 : bit;
	}

	printf("\nChecking rebuilt bitmap....");

	for(p = Q2_XTNT_HDR, lp = Q2_XTNT_HDR, seg = 0 ; lp < bitmap_bytes ; ++lp)
	{
		set_extra_seg(old_bitmap_seg[seg]);
		old_byte = @p ;
		set_extra_seg(new_bitmap_seg[seg]);
		new_byte = @p ;
		if(old_byte != new_byte)
		{
			for(bit = 0x80, i = 8 ; bit ; bit >>= 1, --i)
			{
				if((old_byte & bit) != 0  &&  (new_byte & bit) == 0)
				{
					if(block_read(dp, 8 * (lp - Q2_XTNT_HDR) + i, 1, &bufhdr) == 0)
						++zapped_blks;
					else
						@p |= bit;
				}
				if((old_byte & bit) == 0  &&  (new_byte & bit) != 0)
					++missing_blks;
			}
		}
		if (++p == 0)
			seg++ ;			/* Note rollover!  Won't work with long ptrs! */
	}

	if(zapped_blks)
		printf("\n%ld good blocks marked as used in the bitmap but never used by any file.\n", zapped_blks);
	if(missing_blks)
	{
		printf("\n%ld blocks used by files but marked as free in the bitmap.\n", missing_blks);
		status = min(status, -3);
	}
	if(zapped_blks == 0  &&  missing_blks == 0)
		printf("\nRebuilt bitmap agrees with existing bitmap\n");
	else
	{
		if(fixflag == 0)
			return;

		if(pauseflag)
			query("\n\007Do you wish to write back the rebuilt bitmap to disk? ");


		printf("Writing bitmap back to disk...");
		for(j = 0, seg = 0 ; j < bitmap_blks ; ++j)
		{
			if ((j % 128) == 0)
				set_extra_seg(new_bitmap_seg[seg++]) ;
			for(p = 0 ; p < FBLK_SIZE ; ++p)
				buf[(unsigned) p] = @(p + (j % 128)*FBLK_SIZE);
			if(block_write(dp, (long)(Q2_BITMAP_BLK + j), 1, &bufhdr) != 0)
				error("Unable to write block %ld of bitmap back to disk.\n", (long)(Q2_BITMAP_BLK+j));
		}
	
		putc('\n', stdout);
	}
}


error( char type, char *str )
{
	/*
	 *  0 == no problems
	 * -1 == chkfsys aborted
	 * -2 == fixable error
	 * -3 == fixable with luck and extra work
	 * -4 == not fixable without lots of luck and work
	 */
	static char stat_map[4] = { -4, -2, -3, -3 };

	printf("\nERROR\n%r", &str);
	printf("FILE:%s/%s   XTNT:%u   RBLK:%ld (%lXh)   ABLK:%ld (%lXh)\n\n",
						path, fname, nxtnts, nblks+1, nblks+1, blk, blk); 
	if(frec(stdout))
	{
		printf("\nERROR\n%r", &str);
		printf("FILE:%s/%s   XTNT:%u   RBLK:%u (%Xh)   ABLK:%u (%Xh)\n",
							path, fname, nxtnts, nblks, nblks, blk, blk); 
	}

	status = min(status, stat_map[type - 1]);
	if(pauseflag)
	{
		switch(type)
		{
		case 1:
			printf("This is NOT a fixable error. Type <CR> to continue");
			break;
		case 2:
			if(fixflag)
				printf("This is a fixable error. Type <CR> to fix");
			else
				printf("This is a fixable error. Run chkfsys without -f to fix. Type <CR>");
			break;
		case 3:
			printf("When CHKFSYS finishes, copy this file to another file.\n");
			printf("Then zap this file using the ZAP utility. The copied\n");
			printf("file will be consistant and with luck no data will be lost.\n");
			printf("Type <CR> to continue with chkfsys");
			break;
		case 4:
			printf("You must rewrite the bitmap at the end of CHKFSYS to mark this bit.\n");
			printf("Type <CR> to continue with chkfsys");
			break;
		}

		fflush(stdin);
		getc(stdin);
	}
}



query( char *str )
{
	char buf[5];

	if(pauseflag)
	{
		printf("%s", str);
		fflush(stdin);
		fgetstr(stdin, buf, 4);
		if(strcmp(buf, "yes") == 0  ||  strcmp(buf, "y") == 0)
			return(1);
	}

	putchar('\n');
	exit(status);
}


fix_dir_entry( int type, long value )
{
	FILE *fp;
	char path_buf[128];
	struct dir_entry dir_buf;

	strcat(strcat(strcpy(path_buf, path), "/"), fname);
	if((fp = fopen(path_buf, "mv")) == 0)
	{
		printf("Unable to fix %s\n\n", path_buf);
		return;
	}

	get_dir_entry(fp, &dir_buf);

	switch(type)
	{
	case FIX_LAST_XTNT:
		dir_buf.flast_xtnt = value;
		break;

	case FIX_NUM_XTNTS:
		dir_buf.fnum_xtnt = value;
		break;

	case FIX_NUM_BLKS:
		dir_buf.fnum_blks = value;
		break;

	}

	if(set_dir_entry(fp, &dir_buf) == 0)
		printf("..Fixed\n\n");
	else
		printf("..Unable to fix\n\n");

	fclose(fp);
}


fix_xtnt_header( long blk )
{
	if(block_write(dp, blk, 1, &bufhdr) != 0)
		printf("Unable to fix %s/%s\n\n", path, fname);
	else
		printf("..Fixed\n\n");
}
