
#ifdef __USAGE		/* q2dinit.c */
%C - Initialize a QNX 2 disk(ette)

%C - [-hps] drive
Options:
 -h               Hard disk (must be specified to dinit a hard disk)
 -p               do not Pause
 -s               Suppress initialization of the root directory
                     (use only during failure recovery)
#endif

/*---------------------------------------------------------------------

	$Header: [60]3:/src/util/QRCS/dinit.c.v 2.20 Wed Feb 17 14:59:11 1993 randy Exp $

	$Source: [60]3:/src/util/dinit.c $

	$Log: 	[60]3:/src/util/QRCS/dinit.c.v $
	
	Revision 2.20 Wed Nov 2  14:59:11 1994 terry
	Ported to work under QNX4.2 (producing QNX2.x disks)
	
	Revision 2.20 Wed Feb 17 14:59:11 1993 randy
	2.20
	
	Revision 1.3 11-Jan-89 11:52:49am aboyd
	added qrcs header, after fixing mbuf, dbuf in previous version,
	which caused hard disk to be inited with wrong size
	
---------------------------------------------------------------------*/

#include <libc.h>
#include <sys/types.h>
#include <sys/disk.h>

#include "struct.h"

extern long odtond();

struct _disk_entry		 dbuf;
struct dir_xtnt			*dx;
struct xtnt_hdr			*xh;
struct dir_entry		*dir;
struct disk_desc_blk	*ddb;

long		 size;
int			 dp;
char		 pauseflg = 1,
			 supress_root,
			 hardflag,
			 buf[Q2_FBLK_SIZE],
			*drive;


main( char argc, char *argv[] )
{
	long		 blk, num_blks;
	long		 timeq4, timenow, tzgetit = 0;
	unsigned	 bitmapsize;
	unsigned	 cur_blk;
	int			 opt;
	char		*p;
	char		 bit;

	while( (opt = getopt( argc, argv, "Pshp:k:" )) != -1 )
	{
		switch( opt )
		{
			case 'P':	pauseflg = 0;
						break;
			case 's':	supress_root = 1;
						break;
			case 'h':	hardflag = 1;
						break;
			default:	break;
		}
	}

	if( optind >= argc )
	{
		fprintf( stderr, "%s: no drive specified\n", argv[0] );
		exit( 1 );
	}
	drive = argv[optind++];

	dp = open( drive, O_RDWR );
	if( dp == (-1) )
	{
		perror( drive );
		fprintf( stderr, "%s: unable to open drive\r\l", argv[0] );
		exit( 1 );
	}

	disk_get_entry( dp, &dbuf );

	if( (dbuf.disk_type == _HARD || dbuf.disk_type == _REMOVABLE) && hardflag == 0 )
	{
		fprintf( stderr, "You must specify -h to %s a hard disk.\n", argv[0] );
		exit( 1 );
	}
	else if( dbuf.disk_type == _FLOPPY || dbuf.disk_type == _RAMDISK )
		;
	else
	{
		fprintf( stderr, "%s: Invalid disk type.\n", argv[0] );
		exit( 1 );
	}

	if( pauseflg && dbuf.disk_type != _FLOPPY && dbuf.disk_type != _RAMDISK )
	{
		fprintf(stderr, "\007This drive is not a floppy or ramdisk\n");
		if(hardflag)
		{
			fprintf(stderr, "Continue (y or n) ? ");
			fflush(stdin);
			if(getc(stdin) != 'y')
				exit(-1);
			fprintf(stderr, "Any existing files will be lost! Are you sure (y or n) ? ");
			fflush(stdin);
			if(getc(stdin) != 'y')
				exit(-1);
		}
		else
			exit(-1);
	}

	num_blks = dbuf.num_sectors;

	timeq4 = time( &timenow );			/* get GMT for fseconds field */
	if( timenow <= SECONDS_BETWEEN_1970_AND_1980 )
		timenow = 0;
	else
		timenow -= SECONDS_BETWEEN_1970_AND_1980;
	tzget( &tzgetit );					/* get timezone offset for stuffing fdate */

	if(pauseflg  &&  dbuf.disk_type == _FLOPPY)
	{
		fprintf( stderr, "Place disk in drive '%s' and hit <CR>", drive );
		fflush( stdin );
		getc( stdin );
	}

/*
 * block 1 is the disk descriptor block
 * it is formed as a directory extent with 2 entries
 * the first entry contains the disk_id
 * the 2nd entry points to the root block
 * the name of the 2nd directory is of variable length
 * and represents the disk prefix
 */

	if( block_read( dp, 1, 1, buf ) != 1 )
	{
		fprintf( stderr, "%s: Disk error... Disk may not be formatted\n", argv[0] );
		exit( 1 );
	}

	bitmapsize = ((((num_blks - 1) >> 3) + Q2_XTNT_HDR) >> Q2_FBLK_BITS) + 1;

	if( num_blks < 3 )
	{
		fprintf( stderr, "%s: disk is too small to initialize\n", argv[0] );
		exit( 1 );
	}

	if( pauseflg )
		fprintf( stderr, "Disk contains %ld blocks (%ldK)\n", num_blks, num_blks >> 1 );

	memset( buf, 0, Q2_XTNT_HDR + sizeof(struct dir_entry) + sizeof(struct dir_xtnt) );
	xh = (struct xtnt_hdr *) buf;
	xh->size_xtnt = 2 * sizeof(struct dir_entry) + 4;
	dx = (struct dir_xtnt *) (buf + Q2_XTNT_HDR);
	dx->parent_xtnt = 1;
	dx->dir_index = 1;
	ddb = (struct disk_desc_blk *) (buf+Q2_XTNT_HDR);
	ndtood( timenow+tzgetit, ddb->disk_id );	/* store localtime to disk_id */
	dir = &ddb->disk_root_dir;
	dir->fstat = Q2_USED;
	dir->ffirst_xtnt = dir->flast_xtnt = bitmapsize + 2;
	dir->fnum_blks = 0;
	dir->fnum_xtnt = 1;
	dir->fowner = geteuid();
	dir->fgroup = getegid();
	dir->fperms = Q2_READ | Q2_DIRECTORY;
	dir->fgperms = Q2_READ | Q2_DIRECTORY;
	dir->fattr = Q2_READ | Q2_DIRECTORY | Q2_CREATE | Q2_MODIFY;
	ndtood( timenow+tzgetit, dir->fdate );
	dir->fseconds = timenow;			/* GMT */
	block_write( dp, 1, 1, buf );

/*
 * the next block(s) contain an extent which is the bitmap.
 * used blocks are represented by ones
 */

	memset( buf, 0, Q2_FBLK_SIZE );
	size = ((num_blks - 1) >> 3) + 1;
	xh = (struct xtnt_hdr *) buf;
	xh->size_xtnt = size;
/*
	xh->ext_size_xtnt = size >> 16;
 */
	xh->bound_xtnt = bitmapsize - 1;

	p = buf + Q2_XTNT_HDR;
	bit = 1;
	cur_blk = 2;
	blk = 1;
	while( blk++ <= bitmapsize + 2 )
	{
		*p |= bit;
		if( ! (bit <<= 1) )
		{
			++p;
			bit = 1;
		}
	}

/*
 * mark the remaining blocks as free
 */
	while( blk++ <= num_blks )
	{
		*p &= ~bit;
		if( ! (bit <<= 1) )
		{
			++p;
			bit = 1;
		}
		if( p >= (buf + Q2_FBLK_SIZE) )
		{
			block_write( dp, cur_blk++, 1, buf );
			memset( buf, 0, Q2_FBLK_SIZE );
			p = buf;
		}
	}

/*
 * mark any remaining bits in bitmap as used blocks
 */
	if( ! (p == buf && bit == 1) )
	{
		while( p < (buf + Q2_FBLK_SIZE) )
		{
			*p |= bit;
			if( ! (bit <<= 1) )
			{
				++p;
				bit = 1;
			}
		}
		block_write( dp, cur_blk++, 1, buf );
	}

/*
 * the next block is the root block which initially
 * contains a directory extent with the single file
 * called "bitmap" in it.
 */

	if( ! supress_root )
	{
		memset( buf, 0, Q2_FBLK_SIZE );
		xh = (struct xtnt_hdr *) buf;
		xh->size_xtnt = sizeof(struct dir_entry) + 4;

		dx = (struct dir_xtnt *) (buf + Q2_XTNT_HDR);
		dx->parent_xtnt = 1;
		dx->dir_index = 1;

		dir = &dx->directory[0];
		dir->fstat = Q2_USED;
		dir->ffirst_xtnt = dir->flast_xtnt = 2;
		dir->fnum_blks = bitmapsize - 1;
		dir->fnum_xtnt = 1;
		dir->fowner = 0xff;
		dir->fgroup = 0xff;
		dir->fperms = Q2_READ;
		dir->fgperms = Q2_READ;
		dir->fattr = Q2_READ;
		ndtood( timenow+tzgetit, dir->fdate );
		dir->fseconds = timenow;		/* GMT */
		strcpy( &dir->fname[0], "bitmap" );
		block_write( dp, cur_blk++, 1, buf );
	}
}
