
#ifndef STRUCT_H
#define STRUCT_H

#include <stdio.h>
#include <errno.h>
#include <sys/sys_msg.h>
#include <sys/io_msg.h>
#include <sys/stat.h>
#include <sys/fsys.h>
#include <sys/fsys_msg.h>

#include "define.h"
#include "proto.h"

// The following structures are specific to the QNX 2.x file system and may
//   NOT be modified (if you want a chance in ... of reading that fsys).

struct dir_entry
{
	char			 fstat;
	long			 ffirst_xtnt;
	long			 flast_xtnt;
	long			 fnum_blks;
	unsigned short	 fnum_xtnt;
	unsigned char	 fowner;
	unsigned char	 fgroup;
	unsigned short	 fnum_chars_free; /* No. unused chars in last block */
	long			 fseconds;
	unsigned char	 ftype;
	unsigned char	 fgperms;
	unsigned char	 fperms;
	unsigned char	 fattr;
	unsigned short	 fdate[2];
	char			 fname[Q2_NCPFN + 1];
} ;

struct xtnt_hdr
{
	long			 prev_xtnt;
	long			 next_xtnt;
	long			 size_xtnt;
	long			 bound_xtnt;
} ;

struct disk_desc_blk
{
	unsigned short	 parent_xtnt;
	unsigned short	 dir_index;
	unsigned short	 disk_id[2];
	long			 boot_file_xtnt;
	unsigned short	 boot_spare;
	char			 unused[sizeof(struct dir_entry) - 10];
	struct dir_entry disk_root_dir;
} ;

/*
 * File format of directory
 */

struct dir_xtnt							/* Top bit of dir_index is zero */
{
	unsigned short	 parent_xtnt;
	unsigned short	 dir_index;
	struct dir_entry directory[1];
} ;

struct ext_dir_xtnt						/* Top bit of dir_index is set */
{
	unsigned short	 parent_xtnt;
	unsigned short	 dir_index;
	unsigned short	 ext_parent_xtnt;
} ;


/* These structures may be modified as desired.

   Note that the 'union' described at the end of this file should have the
   first group of structure unions constant. This is the message structure
   that an application will use to talk to the manager.
*/

struct fptr
{
	struct device	*device;
	int				 count;
	int				 lvl;
	long int		 blk;
	int				 off;
	long int		 lxblk;
	long int		 xoff;
	long int		 roff;
	long			 ffirst_xtnt;		// from qnx2 dir_entry
	unsigned short	 fnum_xtnt;			// from qnx2 dir_entry
	unsigned char	 fattr;				// from qnx2 dir_entry
	struct xtnt_hdr	 xh;
	char			 buff[_BLOCK_SIZE];
} ;

#define OCB_RAW		 0x01

struct ocb
{
	mode_t			 mode;				/* Access modes (ie. O_RDWR). */
	char			 count;				/* number of fd's attached to this OCB. */
	char			 flags;
	struct device	*device;
	struct fptr		 fp;
	struct stat		 fstat;				// file stat
} ;

struct device
{
	char			 name[10];			// device name to register
	unsigned char	 namelen;
	int				 fd;
	struct stat		 stat;				// device stat
	char			 type;
} ;

/* This union encompasses all the possible message types that the IO manager
   will receive. The replies are also contained in this union.
   The first paragraph of structures (up to and including 'fstat_reply') 
   will ALWAYS be the same for all IO managers.
*/

union msg
{
	short unsigned					 type;
	short unsigned					 status;
	struct _sysmsg_hdr				 sysmsg;
	struct _sysmsg_hdr_reply		 sysmsg_reply;
	struct _io_open					 open;			/* IO_HANDLE uses this one also.	*/
	struct _io_open_reply			 open_reply;
	struct _io_dup					 dup;				/* Used for fork, exec, dup, dup2()'s. */
	struct _io_dup_reply			 dup_reply;
	struct _io_close				 close;
	struct _io_close_reply			 close_reply;
	struct _io_read					 read;
	struct _io_read_reply			 read_reply;
	struct _io_write				 write;
	struct _io_write_reply			 write_reply;
	struct _io_lseek				 lseek;
	struct _io_lseek_reply			 lseek_reply;
	struct _io_ioctl				 ioctl;
	struct _io_ioctl_reply			 ioctl_reply;
	struct _io_utime				 utime;
	struct _io_utime_reply			 utime_reply;
	struct _io_chmod				 chmod;
	struct _io_chmod_reply			 chmod_reply;
	struct _io_chown				 chown;
	struct _io_chown_reply			 chown_reply;
	struct _io_fstat				 fstat;
	struct _io_fstat_reply			 fstat_reply;
	struct _io_readdir				 readdir;
	struct _io_readdir_reply		 readdir_reply;
	struct _io_rewinddir			 rewinddir;
	struct _io_rewinddir_reply		 rewinddir_reply;
	struct _fsys_block_read			 block_read;
	struct _fsys_block_read_reply	 block_read_reply;
	struct _fsys_block_write		 block_write;
	struct _fsys_block_write_reply	 block_write_reply;

	/* user defined messages after this point */
} ;

#endif
