
#include <libc.h>

#include <sys/psinfo.h>
#include <sys/fd.h>
#include <sys/disk.h>

#include "define.h"
#include "struct.h"
#include "extern.h"


// this function searches the array of control structures looking for a match
// to the path.  It returns -1 on no match and an index into and array for a
// match

int lookup_name( char *nm )
{
	int cnt;

	if( *io_msg.open.path == 0 )		// opening top of dir tree
		return( 0 );
	
	for( cnt = 1; cnt < numdevs; ++cnt )
	{
		if( ! strncmp(nm,device[cnt].name,device[cnt].namelen) )
			return( cnt );
	}
	return( -1 );
}


int map_ocb(int pid, int fd, struct ocb *ocb)
{
	/* Use unit as index into ocb array. */
	if(qnx_fd_attach(pid, fd, 0, 0, 0, 0, (unsigned)ocb) == -1)
	{
		io_msg.open_reply.status = errno;
		return(-1);
	}
	return(0);
}


struct ocb * get_ocb( pid_t pid, int fd)
{
	struct ocb *ocb;
	
	ocb = (struct ocb *)__get_fd(pid, fd, fd_ctrl);
	if(ocb == (struct ocb *)0 || ocb == (struct ocb *)-1)
	{
		io_msg.status = EBADF;
		return((struct ocb *)-1);
	}
	return(ocb);
}


mode_t check_access(pid_t pid, struct stat *sptr)
{
	mode_t mode;
	uid_t uid;
	gid_t gid;

#if 0
ifdef READ_ONLY_FSYS
	mode_t perm_mode[3] = { S_IREAD, 0, 0 };	// read-only permissions
#else
	mode_t perm_mode[3] = { S_IREAD, S_IWRITE, S_IREAD|S_IWRITE };
#endif

										/* Get egid and euid of process pid.*/
	get_ids(pid, &uid, &gid);
					/* Make sure the calling process has correct permissions. */
	mode = perm_mode[io_msg.open.oflag & O_ACCMODE];
	mode &=	(uid == 0) ?
					S_IRWXU : (uid == sptr->st_uid) ?	(sptr->st_mode & S_IRWXU) :
					(gid == sptr->st_gid) ?	(sptr->st_mode & S_IRWXG) << 3 :
						(sptr->st_mode & S_IRWXO) << 6;
	return(mode);
}


void get_ids (pid_t pid, uid_t *uid, gid_t *gid)
{
	struct _psinfo3	 psinfo3;
	
	__get_pid_info(pid, &psinfo3, fd_ctrl);
	*uid = psinfo3.euid;
	*gid = psinfo3.egid;
}


int q2_fget( void *data, int size, struct fptr *fp )
	{
	int num = 0;
	int i;
	long l;
	char *p = (char *)data;

	if( fp->blk == -1 )
		return( 0 );

	if( ! fp->blk )
	{
		fp->blk = fp->ffirst_xtnt;
		if( block_read(fp->device->fd, fp->blk, 1, fp->buff) != 1 )
		{
			fp->blk = -1;
			return( 0 );
		}
		memcpy( &fp->xh, fp->buff, sizeof(struct xtnt_hdr) );
		fp->off = sizeof(struct xtnt_hdr);
		fp->lxblk = fp->blk + fp->xh.bound_xtnt;
		fp->xoff = 0L;
		fp->roff = 0L;
	}

	while( size )
	{
		l = fp->xh.size_xtnt - fp->xoff;
		i = l < INT_MAX ? (int) l : INT_MAX;
		if( i > _BLOCK_SIZE - fp->off )
			i = _BLOCK_SIZE - fp->off;
		if( i > size )
			i = size;
		if( i > 0 )
		{
			if( data )
				memcpy( &p[num], &fp->buff[fp->off], i );
			size -= i;
			num += i;
			fp->off += i;
			fp->xoff += i;
			fp->roff += i;
		}
		if( i <= 0 )
		{
			if( fp->blk == fp->lxblk )
			{
				if( fp->blk = fp->xh.next_xtnt )
				{
					if( block_read(fp->device->fd, fp->blk, 1, fp->buff) != 1 )
					{
						fp->blk = -1;
						return( num );
					}
					memcpy( &fp->xh, fp->buff, sizeof(struct xtnt_hdr) );
					fp->off = sizeof(struct xtnt_hdr);
					fp->lxblk = fp->blk + fp->xh.bound_xtnt;
					fp->xoff = 0L;
				}
				else
				{
					fp->blk = -1;
					return( num );
				}
			}
			else
			{
				++fp->blk;
				if( block_read(fp->device->fd, fp->blk, 1, fp->buff) != 1 )
				{
					fp->blk = -1;
					return( num );
				}
				fp->off = 0;
			}
		}
	}	

	return( num );
}


void load_stat( int st_dev, struct stat *stat, struct dir_entry *de )
{
	time_t clock;

	memset( stat, 0, sizeof(struct stat) );
	stat->st_ino = de->ffirst_xtnt;
	stat->st_dev = st_dev;

	stat->st_size = (de->fnum_blks * _BLOCK_SIZE) + _BLOCK_SIZE
					- (de->fnum_chars_free & 0x1ff)
					- (de->fnum_xtnt * sizeof(struct xtnt_hdr) );

	if(clock = de->fseconds)
		clock += SECONDS_BETWEEN_1970_AND_1980;
	else
		clock = odtond(de->fdate);
	
	stat->st_ftime = clock;
	stat->st_mtime = clock;
	stat->st_atime = clock;
	stat->st_ctime = clock;

	stat->st_uid = (de->fowner == 255) ? 0 : de->fowner;
	stat->st_gid = (de->fgroup == 255) ? 0 : de->fgroup;

	if( de->fattr & Q2_DIRECTORY )
	{
		stat->st_nlink = 2;
		stat->st_mode = S_IFDIR;

		if( ! (de->fattr & Q2_BLOCK) )
			stat->st_mode |= S_IRUSR|S_IXUSR;
		if( ! (de->fgperms & Q2_BLOCK) )
			stat->st_mode |= S_IRGRP|S_IXGRP;
		if( ! (de->fperms & Q2_BLOCK) )
			stat->st_mode |= S_IROTH|S_IXOTH;
	}
	else
	{
		stat->st_nlink = 1;
		stat->st_mode = S_IFREG;

		if( de->fattr & Q2_READ )
			stat->st_mode |= S_IRUSR;
		if( de->fgperms & Q2_READ )
			stat->st_mode |= S_IRGRP;
		if( de->fperms & Q2_READ )
			stat->st_mode |= S_IROTH;

		if( de->fattr & Q2_WRITE )
			stat->st_mode |= S_IWUSR;
		if( de->fgperms & Q2_WRITE )
			stat->st_mode |= S_IWGRP;
		if( de->fperms & Q2_WRITE )
			stat->st_mode |= S_IWOTH;

		if( de->fattr & Q2_EXECUTE )
			stat->st_mode |= S_IXUSR;
		if( de->fgperms & Q2_EXECUTE )
			stat->st_mode |= S_IXGRP;
		if( de->fperms & Q2_EXECUTE )
			stat->st_mode |= S_IXOTH;
	}
		
	if( de->fstat & Q2_USED )
		stat->st_status |= _FILE_USED;
	if( de->fstat & Q2_FILE_MODIFIED )
		stat->st_status |= _FILE_MODIFIED;
	if( de->fstat & Q2_FILE_BUSY )
		stat->st_status |= _FILE_BUSY;
}


int get_dev_stat( struct device *dp )
{
	short unsigned	 date[2];
	time_t			 clock;
	int				 assignum,
					 devno,
					 rdevno;

	if( block_read(dp->fd, 1L, 1, buff) != 1 )
		return( 1 );

	assignum = dp->stat.st_ino;
	devno = dp->stat.st_dev;
	rdevno = dp->stat.st_rdev;

	load_stat( dp->stat.st_dev, &dp->stat, (struct dir_entry *) &buff[0x44] );

	memcpy( date, &buff[0x14], sizeof(date) );
	clock = odtond(date);
	dp->stat.st_ftime = clock;
	dp->stat.st_mtime = clock;
	dp->stat.st_atime = clock;
	dp->stat.st_ctime = clock;

	dp->stat.st_ino = assignum;
	dp->stat.st_dev = devno;
	dp->stat.st_rdev = rdevno;

	return( 0 );
}
